#include <stdint.h>
#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include "lcd.h"

#define SNAKE_START_POS_X 63
#define SNAKE_START_POS_Y 31

#define NO_BUTTON_PRESSED 0
#define SNAKE_MOVE_RIGHT 1
#define SNAKE_MOVE_LEFT 2
#define SNAKE_MOVE_UP 3
#define SNAKE_MOVE_DOWN 4

typedef struct {
    uint8_t x;
    uint8_t y;
} Point;

void init_buttons(void) {

    // initializari pentru pinii folositi
    DDRD |= (1 << PD0);
    DDRD |= (1 << PD1);
    DDRD |= (1 << PD4);
    DDRD |= (1 << PD5);

    PORTD &= ~(1 << PD0);
    PORTD &= ~(1 << PD1);
    PORTD &= ~(1 << PD4);
    PORTD &= ~(1 << PD5);

    // setare directie pini(citire) si activare rezistenta pull-up
    // UP
    DDRD &= ~(1 << PD0);
    PORTD |= (1 << PD0);
    
    // LEFT
    DDRD &= ~(1 << PD1);
    PORTD |= (1 << PD1);
    
    // RIGHT
    DDRD &= ~(1 << PD4);
    PORTD |= (1 << PD4);
    
    // DOWN
    DDRD &= ~(1 << PD5);
    PORTD |= (1 << PD5);
}

uint8_t check_input(void) {
    uint32_t i;
    uint8_t button_pressed = NO_BUTTON_PRESSED;

    // polling la butoane & waiting
    for(i = 100000; i > 0; i--) {
        if(!button_pressed) {
            // DOWN
            if (!(PIND & (1 << PD0))) {
                button_pressed = SNAKE_MOVE_DOWN;
            }
            
            // LEFT
            if (!(PIND & (1 << PD1))) {
                button_pressed = SNAKE_MOVE_LEFT;
            }
            
            // RIGHT
            if (!(PIND & (1 << PD4))) {
                button_pressed = SNAKE_MOVE_RIGHT;
            }
            
            // UP
            if (!(PIND & (1 << PD5))) {
                button_pressed = SNAKE_MOVE_UP;
            }
        }
    }

    return button_pressed;
}

inline void set_block(uint8_t x, uint8_t y, uint8_t v) {
    lcd_setbit(x, y, v);
    lcd_setbit(x, y + 1, v);
    lcd_setbit(x + 1, y, v);
    lcd_setbit(x + 1, y + 1, v);
}

void init_snake(void) {
    set_block(SNAKE_START_POS_X, SNAKE_START_POS_Y, 1);
    set_block(SNAKE_START_POS_X + 2, SNAKE_START_POS_Y, 1);
    set_block(SNAKE_START_POS_X + 4, SNAKE_START_POS_Y, 1);

    lcd_flush();
}

int main(void) {
    lcd_init();

    init_buttons();

    init_snake();

    Point snake_head;
    snake_head.x = SNAKE_START_POS_X + 4;
    snake_head.y = SNAKE_START_POS_Y;

    uint8_t move_dir = SNAKE_MOVE_RIGHT;
    uint8_t new_move_dir = NO_BUTTON_PRESSED;

    while(1) {
        new_move_dir = check_input();
        if(new_move_dir != NO_BUTTON_PRESSED)
            move_dir = new_move_dir;

        switch(move_dir) {
            case SNAKE_MOVE_RIGHT: {
                set_block(snake_head.x + 2, snake_head.y, 1);
                snake_head.x += 2;
                break;
            }
            case SNAKE_MOVE_UP: {
                set_block(snake_head.x, snake_head.y - 2, 1);
                snake_head.y -= 2;
                break;
            }
            case SNAKE_MOVE_DOWN: {
                set_block(snake_head.x, snake_head.y + 2, 1);
                snake_head.y += 2;
                break;
            }
            case SNAKE_MOVE_LEFT: {
                set_block(snake_head.x - 2, snake_head.y, 1);
                snake_head.x -= 2;
                break;
            }
        }
        lcd_flush();
        _delay_ms(100);
    }
  return 0;
}